package bitel.billing.test.convert;

import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import bitel.billing.common.CommonTimeUtils;
import bitel.billing.common.CommonUtils;
import bitel.billing.server.contract.bean.Contract;
import bitel.billing.server.contract.bean.ContractManager;
import bitel.billing.server.contract.bean.ContractMemo;
import bitel.billing.server.contract.bean.ContractMemoManager;
import bitel.billing.server.contract.bean.ContractParamUtils;
import bitel.billing.server.util.DefaultServerSetup;
import bitel.billing.server.util.Utils;

public class Magadan
{
    private DefaultServerSetup bgSetup;
	private DefaultServerSetup utm1Setup;
	
	private final int PID_EMAL = 11;
	private final int PID_INN = 23;
	private final int PID_KPP = 24;
	private final int PID_FIO = 3;
	private final int PID_PASSPORT = 14;
	private final int PID_TELEPHONE = 5;
	
	
    
	
	private Calendar startMonth = new GregorianCalendar( 2007, Calendar.DECEMBER, 1 );
    
    public static void main(String[] args) 
    {
        new Magadan();        
    }
	
	public Magadan()
	{
		try
		{
			System.out.println( "importing...");
			
			bgSetup = new DefaultServerSetup();
            bgSetup.setValue( "db.driver", "com.mysql.jdbc.Driver" );            								
            bgSetup.setValue( "db.url", "jdbc:mysql://127.0.0.1/bgbilling?useUnicode=true&characterEncoding=Cp1251&allowUrlInLocalInfile=true&zeroDateTimeBehavior=convertToNull&dbcCompliantTruncation=false" );
            bgSetup.setValue( "db.user", "bill" );
            bgSetup.setValue( "db.pswd", "bgbilling" );
            bgSetup.initConnectionPool();
            
            utm1Setup = new DefaultServerSetup();
            utm1Setup.setValue( "db.driver", "com.mysql.jdbc.Driver" );
            utm1Setup.setValue( "db.url", "jdbc:mysql://10.1.1.248/UTM5?useUnicode=true&characterEncoding=utf8&allowUrlInLocalInfile=true&zeroDateTimeBehavior=convertToNull&jdbcComplian" );
            utm1Setup.setValue( "db.user", "bgbilling" );
            utm1Setup.setValue( "db.pswd", "bgbilling" );
            utm1Setup.initConnectionPool();
            
            
			/*
			bgSetup = new DefaultServerSetup();
            bgSetup.setValue( "db.driver", "com.mysql.jdbc.Driver" );            								
            bgSetup.setValue( "db.url", "jdbc:mysql://127.0.0.1:3307/bgbilling?useUnicode=true&characterEncoding=Cp1251&allowUrlInLocalInfile=true&zeroDateTimeBehavior=convertToNull&dbcCompliantTruncation=false" );
            bgSetup.setValue( "db.user", "bill" );
            bgSetup.setValue( "db.pswd", "bgbilling" );
            bgSetup.initConnectionPool();
            
            utm1Setup = new DefaultServerSetup();
            utm1Setup.setValue( "db.driver", "com.mysql.jdbc.Driver" );
            utm1Setup.setValue( "db.url", "jdbc:mysql://127.0.0.1/UTM5?useUnicode=true&characterEncoding=utf8&allowUrlInLocalInfile=true&zeroDateTimeBehavior=convertToNull&jdbcComplian" );
            utm1Setup.setValue( "db.user", "bgbilling" );
            utm1Setup.setValue( "db.pswd", "bgbilling" );
            utm1Setup.initConnectionPool();
            
			*/
				
            
            int dialupID  = 5;
            int cardId = 3;
            int group = 64;
            
            clearDatabase( dialupID, group );
            
            moveUsers( utm1Setup, dialupID, group);            
            moveCards( utm1Setup, cardId );
            
            System.out.println( "finished..");
            
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
    
	private  static Map<String, Integer >  getBillingTariffMap( DefaultServerSetup setup )
	{
		Map<String, Integer > tarifMap = new HashMap<String, Integer>();
		Connection con = null;
		try
		{
			con = setup.getDBConnectionFromPool();
			
			String query = "SELECT id, title FROM tariff_plan";
			
			 
			
			PreparedStatement ps =  con.prepareStatement(   query );            
			ResultSet rs = ps.executeQuery();
			while ( rs.next() )
			{
				tarifMap.put( rs.getString( 2 ), rs.getInt( 1 )  );
				
			}
			rs.close();
		} catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			setup.returnConToPool( con );
		}
		
		
		
		return tarifMap; 
	}
		
    private void clearDatabase( int dialupID, int group)
    {
        Connection bgCon = bgSetup.getDBConnectionFromPool();
        
        try 
        {
            Statement st = bgCon.createStatement();
	    st.executeUpdate( "delete from contract where gr & " + group + " != 0" );
            st.executeUpdate( "delete from contract_balance where cid not in (select id from contract)" );
            st.executeUpdate( "delete from contract_service where cid not in (select id from contract)" );
	    st.executeUpdate( "delete from contract_account where cid not in (select id from contract) ");
            st.executeUpdate( "delete from user_login_" + dialupID + " where cid not in (select id from contract)" );
	    st.executeUpdate( "delete from user_alias_" + dialupID + " where login_id not in (select id from user_login_" + dialupID + " where cid in (select id from contract where gr<>1))" );
	    st.executeUpdate( "delete from radius_pair_ip_" + dialupID + " where id not in (select id from user_login_" + dialupID + " where cid in (select id from contract where gr<>1))" );
            st.executeUpdate( "delete from contract_comment where cid not in (select id from contract)" );
            st.executeUpdate( "delete from contract_parameter_type_2 where cid not in (select id from contract)" );
        } 
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            bgSetup.returnConToPool( bgCon );
        }
    }
    
    private static final int PATTERN_JUR = 2;
    private static final int PATTERN_PHIS = 1;
    
    private int loginNum;
    private Set<String> aliasSet = new HashSet<String>();
    
    private void moveUsers( DefaultServerSetup utmSetup, int dialupID, int group )
    {
        Connection bgCon = bgSetup.getDBConnectionFromPool();
        Connection utmCon = utmSetup.getDBConnectionFromPool();
        
        
        try 
        {
            
	    	//utmCon.createStatement().executeUpdate( "SET NAMES 'latin1'" );
	    	
	    	
         
	  
            
			Map<Long, Set> userIP = getUserIPMap( utmCon );
	
			Map<String, Integer> billingTariffMap = getBillingTariffMap( bgSetup );
			
			Map<Integer, Set> userTarifMap = getUserTariffMap( utmCon );
			
			Map<Integer, String> UTMTariffMap = getUTMTariffsMap( utmCon );
			
				 
	    
	 
	    	    
														     
        
        
        	ContractManager cm = new ContractManager( bgCon );
        
        
			
			String query = 
			    "SELECT users.*, accounts.balance, accounts.credit " +
			    "FROM users " +
			    "LEFT JOIN accounts ON users.basic_account=accounts.id " +                
			    "WHERE users.is_deleted=0 and accounts.unlimited=0";        
        
        
	        ResultSet rs = utmCon.createStatement().executeQuery( query );
	        while( rs.next() )
	        {                
	    		int userId = rs.getInt( "id" );
	            String login = readString(rs,  "login" );
	            String password = rs.getString( "password" );
	            
	          
	            
	           	String fullName = readString(rs,  "full_name" );
	            int basicAccount = rs.getInt( "basic_account" );
	            
	            boolean isJur = rs.getBoolean( "is_juridical" );
	            
	                  
	            System.out.println( "inserting login:" + login + ";userId=" + userId );
	            
	            System.out.println( "fullName:" + fullName );
	            
	            
	            try
				{		
	                Contract contract =
	                    cm.createFromPattern( isJur ? PATTERN_JUR : PATTERN_PHIS, 
	                                                String.valueOf( basicAccount ), 
	                                                startMonth, password );
	                int cid = contract.getID();
	                
	                
	
	                updateAdress( bgCon, rs, cid );
	                updateContact( group, bgCon, rs, cid );                                
	                updateBalance( bgCon, rs, cid );                
	                addContactParams( bgCon, rs, cid );                
	                updateComments( bgCon, cid, rs );
	
	                
	        	    
	                login = addDialupInfo( dialupID,
							bgCon,							
							cid,
							utmCon,
							rs,
							userId,
							userIP);
					
	
	                updateTariffs( bgCon,
							utmCon,
							userId,
							login,
							cid,
							billingTariffMap,
							userTarifMap,
							UTMTariffMap
							);
					
					
					
					
				
				} catch (Exception e)
				{					
					System.out.println( "error !! userId =" + userId + ";login=" + login);
					e.printStackTrace();
				}
				
	        }
        } 
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            bgSetup.returnConToPool( bgCon );
            utmSetup.returnConToPool( utmCon );
        }
    }

	private Map<Long, Set> getUserIPMap( Connection utmCon ) throws SQLException
	{
		String query;
		query = "SELECT ip_groups.ip , service_links.user_id  FROM ip_groups " + 
		" LEFT JOIN iptraffic_service_links ON ip_groups.ip_group_id = iptraffic_service_links.ip_group_id " +
		" LEFT JOIN service_links ON iptraffic_service_links.id = service_links.id " + 

		" WHERE ip_groups.is_deleted = 0 " +
		" AND " + 
		"service_links.is_deleted = 0";		   
		
		PreparedStatement psSelectIpAddress = utmCon.prepareStatement( query );
		ResultSet rs = psSelectIpAddress.executeQuery();
		Map<Long, Set> userIP = new HashMap<Long, Set> ();
		while (rs.next())
		{
			long  ip = rs.getLong( 1 );
			long  uid = rs.getLong( 2 );
			
			
			
			Set ips = userIP.get( uid );
			
			if (ips == null)
			{
				ips = new HashSet ();
				
			}
			
			ips.add( ips );
		}
		return userIP;
	}

	private void updateTariffs( Connection bgCon,
								Connection utmCon,
								int userId,
								String login,
								int cid,
								Map<String, Integer> billingTariffMap,
								Map<Integer, Set> userTarifMap,
								Map<Integer, String> UTMTariffMap ) throws SQLException
	{
		String query;

		
		
		query = "INSERT INTO contract_tariff( cid, tpid) VALUES ( ?, ?)";
		PreparedStatement psInsertTariff =  bgCon.prepareStatement(   query );                
		
		////tariffs
		Set<Integer> tariffs = userTarifMap.get( userId );
		if (tariffs != null )
		{
			for (Integer tarifId : tariffs)
			{
				
			
				String tariffName = UTMTariffMap.get( tarifId );
				
				Integer tpid = billingTariffMap.get( tariffName );
									
				if ( tpid != null)
				{
					psInsertTariff.setInt( 1, cid );
					psInsertTariff.setInt( 2, tpid );
					psInsertTariff.executeUpdate();
				}
				else
				{
					System.out.println( "error !! userId =" + userId + ";login=" + login  + "; tariff \"" + tariffName + "\" was not found!!! ");
				}
			}
		}
	}

	private String addDialupInfo(	int dialupID,
									Connection bgCon,																											
									int cid,
									Connection utmCon,
									ResultSet userRS,
									int userId,
									Map<Long, Set> userIP) throws SQLException, UnsupportedEncodingException
	{
		
		int blocked = userRS.getInt( "is_blocked" );
        String login = readString(userRS,  "login" );
        String password = userRS.getString( "password" );
        
		
		
		String query =
		    "SELECT id FROM user_login_" + dialupID + " WHERE login=? LIMIT 1";
		PreparedStatement psSelectLogin = bgCon.prepareStatement( query );

		
		loginNum++;
		 
		while( true )
		{               
		    psSelectLogin.setLong( 1, loginNum );
		    psSelectLogin.executeQuery();
		    ResultSet rs2 = psSelectLogin.executeQuery();
							                         
		    if( rs2.next() )
		    {
			loginNum++;
		    }
		    else
		    {
			rs2.close();
		        break;
		    }
		}

		query = 
		    "INSERT INTO user_login_" + dialupID + " (cid, login, pswd, date1, status) VALUES (?, ?, ?, ?, ?)";
		PreparedStatement psInsertLogin = bgCon.prepareStatement( query );
						
		psInsertLogin.setInt( 1, cid );
		psInsertLogin.setInt( 2, loginNum );
		psInsertLogin.setString( 3, password );
		psInsertLogin.setDate( 4, CommonTimeUtils.convertCalendarToSqlDate( startMonth ) );
		psInsertLogin.setInt( 5, blocked );
		psInsertLogin.executeUpdate();
		
		
		
		int lid = Utils.lastInsertId( psInsertLogin );
		if( Utils.parseLongString( login ) > 0 )
		{
		    login = login + "a";
		}
		
		String loginWithAdd = login;
		
		
		query =
	        "SELECT login_alias FROM user_alias_" + dialupID + " WHERE login_alias=? LIMIT 1";
	    PreparedStatement psSelectLoginAlias = bgCon.prepareStatement( query );		

		int add = 1;
		while( true )
		{                
		    psSelectLoginAlias.setString( 1, loginWithAdd );
		    psSelectLoginAlias.executeQuery();
		    ResultSet rs3 = psSelectLoginAlias.executeQuery();
							                         
		    if( rs3.next() )
		    {
//                	loginWithAdd = login + "_" + (add++);
		        	loginWithAdd = login + "9";
		    }
		    else
		    {
			rs3.close();
		        break;
		    }
		}
		
		
		login = loginWithAdd;
		aliasSet.add( login );
		
		
		query = 
		    "INSERT INTO user_alias_" + dialupID + " (login_id, login_alias) VALUES (?, ?)";
		PreparedStatement psInsertAlias = bgCon.prepareStatement( query );
		              
		psInsertAlias.setInt( 1, lid );
		psInsertAlias.setString( 2, login );
		psInsertAlias.executeUpdate();
		
        
				
		
		
		Set<Long> ips = userIP.get( userId );
		
		
		query = 
		    "INSERT INTO radius_pair_ip_" + dialupID + " (id, ip, realm) VALUES (?, ?, 'default')";
		PreparedStatement psInsertIpAddrss = bgCon.prepareStatement( query );				
		if (ips != null)
		{
			for (Long ip :  ips )
			{
			    psInsertIpAddrss.setInt( 1, lid );
			    psInsertIpAddrss.setLong( 2, ip );
			    psInsertIpAddrss.executeUpdate();
			}
		}
		return login;
	}
	
   
    
    
	private void updateAdress( Connection bgCon, ResultSet rs, int cid ) throws UnsupportedEncodingException,
																		SQLException
	{
		String query;
		String floor = readString(rs,  "floor" );
		String entered = readString(rs,  "entrance" );
		String actualAddress = readString(rs, "actual_address" );
		String flatNumber = readString(rs,  "flat_number" );
		String jurAddress = readString(rs, "juridical_address" );
		
		query =
		    "INSERT INTO contract_parameter_type_2 (cid, pid, flat, pod, floor, address) VALUES (?, ?, ?, ?, ?, ?)";
		PreparedStatement psInsertAddressParam = bgCon.prepareStatement( query );                
		psInsertAddressParam.setInt( 1, cid );
		
		                
		if( CommonUtils.notBlankString( jurAddress ) )
		{
		    psInsertAddressParam.setInt( 2, 2 );
		    psInsertAddressParam.setString( 3, "" );
		    psInsertAddressParam.setNull( 4, Types.INTEGER );
		    psInsertAddressParam.setNull( 5, Types.INTEGER );
		    psInsertAddressParam.setString( 6, jurAddress );
		    psInsertAddressParam.executeUpdate();
		}
		if( CommonUtils.notBlankString( actualAddress ) )
		{
		    psInsertAddressParam.setInt( 2, 3 );
		    psInsertAddressParam.setString( 3, flatNumber );
		    psInsertAddressParam.setInt( 4, CommonUtils.parseIntString( entered ) );
		    psInsertAddressParam.setInt( 5, CommonUtils.parseIntString( floor ) );
		    psInsertAddressParam.setString( 6, actualAddress );
		    psInsertAddressParam.executeUpdate();
		}
	}

	private void updateComments( Connection bgCon, int cid, ResultSet rs) throws UnsupportedEncodingException, SQLException
	{
		String comments = readString(rs,  "comments" );
		ContractMemoManager memoManager = new ContractMemoManager( bgCon );
		if( CommonUtils.notBlankString( comments ) )
		{
		    ContractMemo memo = new ContractMemo();
		    memo.setUserId( 0 );
		    memo.setContractId( cid );
		    memo.setComment( comments );
		    memo.setSubject( "" );
		    memoManager.updateMemo( memo );
		}
	}

	private void updateBalance( Connection bgCon, ResultSet rs, int cid ) throws SQLException
	{
		String query;
		query = 
		    "INSERT INTO contract_balance ( cid, summa1, yy, mm) VALUES (?, ?, ?, ?)";
		PreparedStatement psInsertBalance = bgCon.prepareStatement( query );
		psInsertBalance.setInt( 3, startMonth.get( Calendar.YEAR ) );
		psInsertBalance.setInt( 4, startMonth.get( Calendar.MONTH ) + 1 );
		
		double balance = rs.getDouble( "balance" );
		psInsertBalance.setInt( 1, cid );
		psInsertBalance.setDouble( 2, balance );
		psInsertBalance.executeUpdate();
	}

	private void updateContact( int group,
								Connection bgCon,
								ResultSet rs,
								int cid ) throws SQLException, UnsupportedEncodingException
	{
		String query;
		String fullName = readString(rs,  "full_name" );
		query = 
		    "UPDATE contract SET closesumma=?, comment=?, gr=gr| ? WHERE id=?";
		PreparedStatement psUpdateContract = bgCon.prepareStatement( query );
		                
		
		
		double credit = rs.getDouble( "credit" );
		                
		psUpdateContract.setDouble( 1, credit );
		psUpdateContract.setString( 2, fullName );
		psUpdateContract.setInt( 3, group );
		psUpdateContract.setInt( 4, cid );
		psUpdateContract.executeUpdate();
	}

	private void  addContactParams(	Connection bgCon,
										ResultSet rs,
										int cid )	throws UnsupportedEncodingException,
													SQLException
	{
		String workTelephone = readString(rs,  "work_telephone" );
		String homeTelephone = readString(rs,  "home_telephone" );
		String mobileTelephone = readString(rs,  "mobile_telephone" );
		
		String telephones = combineTelehones( workTelephone,
				homeTelephone,
				mobileTelephone );
		
		String innNumber = readString(rs,  "tax_number" );
		String kppNumber = readString(rs,  "kpp_number" );
		String email = readString(rs,  "email" );		
		String passport = readString(rs,  "passport" );		
		String personalManager = readString(rs,  "personal_manager" );               
		boolean isSendInvoice = rs.getBoolean( "is_send_invoice" );
		String fullName = readString(rs,  "full_name" );
		
		
				
		
		
		ContractParamUtils cpu = new ContractParamUtils( bgCon );
		cpu.setStringParam( cid, PID_FIO, fullName );
		
		if( CommonUtils.notBlankString( telephones ) )
		{
		    cpu.setStringParam( cid, PID_TELEPHONE, workTelephone );
		}
	
		if( CommonUtils.notBlankString( innNumber ) )
		{
			cpu.setStringParam( cid, PID_INN, innNumber );
		}
		if( CommonUtils.notBlankString( kppNumber ) )
		{
		    cpu.setStringParam( cid, PID_KPP, kppNumber );
		}
		
		if( CommonUtils.notBlankString( passport ) )
		{
		    cpu.setStringParam( cid, PID_PASSPORT, passport );
		}
		/*if( CommonUtils.notBlankString( personalManager ) )
		{
		    cpu.setStringParam( cid, 11, personalManager );
		}
		
		cpu.setFlagParam( cid, 12, isSendInvoice );
		*/
		if( CommonUtils.notBlankString( email ) )
		{
		    cpu.setEmailParam( cid, PID_EMAL, email, "", "" ); 
		}
		
	}

	private String combineTelehones(	String workTelephone,
										String homeTelephone,
										String mobileTelephone )
	{
		String telephones = "";
		boolean flag = false;
		if ( !Utils.isEmptyString( workTelephone ) )
		{
			telephones += workTelephone;
		}
		
		if ( !Utils.isEmptyString( homeTelephone ) )
		{
			if ( flag )
			{
				telephones += ",";
			}
				
			telephones += workTelephone;
		}
		 
		if ( !Utils.isEmptyString( mobileTelephone ) )
		{
			if ( flag )
			{
				telephones += ",";
			}
				
			telephones += workTelephone;
		}
		return telephones;
	}

	private Map<Integer, Set> getUserTariffMap( Connection utmCon ) throws SQLException
	{
		ResultSet rs;
		Map<Integer, Set> userTarifMap = new HashMap<Integer, Set> (); 
	    
    	String query2 = 
            "SELECT service_links.user_id as userId, tariffs.id as tariffId " +
            "FROM service_links " +                        
            "LEFT JOIN  tariffs_services_link ON service_links.service_id=tariffs_services_link.service_id " +
            "LEFT JOIN  tariffs ON tariffs_services_link.tariff_id=tariffs.id " +
            "LEFT JOIN  services_data ON service_links.service_id=services_data.id " +
            "WHERE tariffs_services_link.is_deleted=0 " +
            "AND services_data.is_deleted=0 " +
            "AND service_links.is_deleted=0 " +
    		"AND tariffs.is_deleted=0 ";
                	
    	
    	
            
        rs = utmCon.createStatement().executeQuery( query2 );
        while ( rs.next() )
        {
        	int userId = rs.getInt(1);
        	int tarifId = rs.getInt( 2 ); 
        	
        	if (tarifId > 0)
        	{
        		Set tariffs =   userTarifMap.get( userId );
        		if (tariffs == null)
        		{
        			tariffs = new HashSet();
        			userTarifMap.put( userId, tariffs );
        			
        		}
        		
        		tariffs.add( tarifId );
        	}
        	
        }
		return userTarifMap;
	}
	private Map<Integer, String> getUTMTariffsMap( Connection utmCon ) throws SQLException
	{
		ResultSet rs;
		Map<Integer, String> result = new HashMap<Integer, String> (); 
	    
    	try
		{
			String query2 = 
			    "SELECT id, name " +
			    "FROM tariffs ";
			
			rs = utmCon.createStatement().executeQuery( query2) ;
			
			while ( rs.next() )
			{
				int userId = rs.getInt(1);
				String tarifName = readString(rs, "name"); 
				
				result.put( userId, tarifName );
				
				
			}
		} catch (Exception e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}
    
    
    private void moveCards( DefaultServerSetup utmSetup, int cardId )
    {
        Connection bgCon = bgSetup.getDBConnectionFromPool();
        Connection utmCon = utmSetup.getDBConnectionFromPool();
        try 
        {
            String query = null;
            
            query = "DELETE FROM card_data_" + cardId;
            bgCon.createStatement().executeUpdate( query );
            
            query = "SELECT id, secret, balance FROM card_info WHERE is_used=0";
            PreparedStatement psSelect = utmCon.prepareStatement( query );
            
            query = "INSERT INTO card_data_" + cardId + " (id, card_code, card_pin_code, summa, date1, status, sid_pay) VALUES (?, ?, ?, ?, '2007-12-01', 0, 1)";
            PreparedStatement psInsert = bgCon.prepareStatement( query );
            
            ResultSet rs = psSelect.executeQuery();
            while( rs.next() )
            {
                int login = rs.getInt( 1 );
                String pswd = rs.getString( 2 );
                String balance = rs.getString( 3 );
                
                psInsert.setInt( 1, login );
                psInsert.setInt( 2, login );
                psInsert.setString( 3, pswd );
                psInsert.setString( 4, balance );
                psInsert.executeUpdate();
            }
            
        } 
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            bgSetup.returnConToPool( bgCon );
            utmSetup.returnConToPool( utmCon );
        }
    }
    
    private static String readString( ResultSet resultSet, String field ) throws UnsupportedEncodingException, SQLException
	{
		
    	/*byte [] bytes = resultSet.getBytes( field );
    	return bytes == null ? null : new String( bytes, "utf-8" );
    	*/
    	return resultSet.getString( field );
	}   
}