package bitel.billing.test.convert;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.Reader;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import bitel.billing.common.CommonTimeUtils;
import bitel.billing.common.CommonUtils;
import bitel.billing.server.admin.resource.bean.NumberResource;
import bitel.billing.server.admin.resource.bean.NumberResourceUseManager;
import bitel.billing.server.contract.bean.Contract;
import bitel.billing.server.contract.bean.ContractManager;
import bitel.billing.server.phone.bean.ClientItem;
import bitel.billing.server.phone.bean.ClientItemManager;
import bitel.billing.server.script.bean.event.EventProcessor;
import bitel.billing.server.util.BaseException;
import bitel.billing.server.util.DefaultServerSetup;
import bitel.billing.server.util.Utils;

public class Mnu 
{
	private static DefaultServerSetup bgSetup = null;
	private static int GROUP_ID = 1;
	private static int PHYS_PATTERN = 2;
	private static int JUR_PATTERN = 1;
	//private static int SOURCE_ID = 1;
	private static int MODULE_ID = 1;
	
	private static int MD110_ID = 1;
	private static int SI2000_ID = 2;
	
	private static String MD110_LIST [] = { "LIM-8", "LIM-9", "LIM-10", "LIM-11", "-110" };  
	private static String SI2000_LIST [] = {"BP-250",  "DX-500", "SI-2000", "Simens", "Si-2000", "ip-.", "LIM "  };
	
	private static Set<String> MD110_SET = new HashSet<String>();
	private static Set<String> SI2000_SET = new HashSet<String>();
	
	static{
		
		for (String value : MD110_LIST)
		{
			MD110_SET.add( value );
		}
		
		for (String value : SI2000_LIST)
		{
			SI2000_SET.add( value );
		}		
	}
	
	
	
	
	public static List< Map >  loadData( String filePath) throws IOException
	{
		List< Map > result = new ArrayList<Map>();
		Reader reader =  new java.io.InputStreamReader( new FileInputStream( filePath  ), "utf8" );
		 
		String[][] lines = com.Ostermiller.util.ExcelCSVParser.parse(  reader, ';' );
		
		for ( int i = 1 ; i < lines.length; i++)
		{
			String [] line =  lines[i];
			Map map = new HashMap();
			
			map.put( "number", line [0] );
			map.put( "ats",  line [1]  );			
			map.put( "gorod",  line [5] );
			map.put( "address",  line [6] );
			map.put( "firm",  line [8] );
			map.put( "abonent", line [11] );
			map.put( "name", line [12] );
			map.put( "comment", line [13] );
			result.add( map );			
			
		}
		
		return result;
	}
	
	
	
	private static Map < Integer, Contract> createContacts( List<Map> numbers )
	{
		
		Map < Integer, Contract> result = new HashMap<Integer, Contract>();
		Connection con = null;
		Date currDate = new Date();
		
		Map<String, Contract> contactMap = new HashMap<String, Contract>();
		try
		{
			con = bgSetup.getDBConnectionFromPool();
			
			ContractManager manager =  new ContractManager( con );
			
			System.out.println( numbers.size() + " numbers was found..");		
			
			int idx = 0;
			for ( Map<String, String> map : numbers)
			{				
	    		Date date = new Date();
	    		
	    		int pattern = 0;
	    		
	    		String title =map.get( "firm" );
	    		
	    		Contract contract = null;
	    		
	    		boolean createFirm = false; 
	    		boolean createFlat = false;
	    		
	    		if (CommonUtils.isEmptyString( title ))
	    		{
	    			continue;
	    		}
	    		else if (title.equals( "" ))
	    		{
	    			pattern = PHYS_PATTERN;
	    			title = map.get( "abonent" );
	    			createFlat = true;
	    			
	    		}
	    		else if (  ( contract = contactMap.get( title ) ) == null)
	    		{
	    			pattern = JUR_PATTERN;
	    			createFirm = true;
	    		}
	    		
	    			    		
	    		if ( createFlat || createFirm )
	    		{
	    			contract = manager.createFromPattern( pattern, null, CommonTimeUtils.convertDateToCalendar( date ), "");
	    		}
	    		
	    		if (contract != null)
	    		{	
	    			int cid = contract.getID();
	    			
	    			map.put( "cid", String.valueOf( cid ) );
	    			
	    			result.put( cid, contract );	
	    			
	    			
	    			if ( createFlat || createFirm  )
	    			{	    			
						updateContactInfo( cid, map, title );
	    			}
					
					if (createFirm)
					{
						contactMap.put( title, contract );
					}
					
	    		}
	    		
	    		
	    		        		
	    	}			
			
			System.out.println( result.size() + " contracts was added..");
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{
			bgSetup.returnConToPool( con );
		}		
		
		return result;		
	}	
	
	private static void updateContactInfo( int cid, Map<String, String> map, String title )
	{
		Connection con = null;
		
		try
		{
			con = bgSetup.getDBConnectionFromPool();
			String query = "UPDATE contract SET gr=gr| ?,  " +						
						" comment = ? " +
					" WHERE id=?";
			PreparedStatement ps = con.prepareStatement( query );
			int idx =1;
			ps.setInt( idx++, GROUP_ID );			
			ps.setString( idx++, title );
			ps.setInt( idx++, cid );

			ps.executeUpdate();
			
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{
			bgSetup.returnConToPool( con );
		}		
		
	}

	private static void createPoints(	List<Map> numbers,
										Map<Integer, Contract> contracts ) throws BaseException
	{
		Connection con = null;
		try
		{
			con = bgSetup.getDBConnectionFromPool();
			
			int pc = 0;
			
			for (Map<String, String> map : numbers)
			{
				ClientItem item = new ClientItem();

				
				if (map.get( "cid" ) == null)
				{
					continue;
				}
				
				Integer cid = Integer.valueOf( map.get( "cid" ) );

				String number = "738259" + map.get( "number" ) ;
				String ats = map.get( "ats" );
				
				int sourceId = 0;

				if (!CommonUtils.isEmptyString( ats ))
				{
					if ( MD110_SET.contains( ats ) )
					{
						sourceId = MD110_ID;
					}
					else if ( SI2000_SET.contains( ats ))
					{
						sourceId = SI2000_ID;
					}
					else
					{
						System.out.println( " error : unknown ATS " + ats + "!!!!!!!!!!!" + "for number " + number);
						continue;
					}
					
				}
				else
				{
					System.out.println( " error : empty ATS!!! for number " + number);
					continue;
				}
				
				pc++;
				item.setContractId( cid );
				item.setSourceId( sourceId );

				item.setDate1( new GregorianCalendar() );

				item.setType( ClientItem.TYPE_POINT );

				item.setClientNumbers( Utils.stringToList( number ) );
				item.setClientPorts( new ArrayList<String>() );
				
				item.setComment( "   " );
				item.setAlias( "" );
				 
				ClientItemManager man = new ClientItemManager( con,
						MODULE_ID );
				NumberResourceUseManager resourceUseManager = new NumberResourceUseManager( con,
						MODULE_ID );

				man.addClientItem( item );
				
				

				/*
				 *     //    
				 *      Map<String,
				 * NumberChange> numberMap = new HashMap<String,
				 * NumberChange>();
				 * 
				 * for( NumberChange change : numberMap.values() ) { if(
				 * change.resource == null ) { continue; }
				 * 
				 * List<NumberResourceUse> useList = man.getResourceUse(
				 * change.resource ); resourceUseManager.updateResourceUse(
				 * change.resource.getId(), useList ); }
				 */
			}
			
			System.out.println( pc + " points was added" );
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			throw new RuntimeException("Error while creating points");
		}

		finally
		{
			bgSetup.returnConToPool( con );
		}

	}
	
	public static void main( String[] args )
	{
		try
		{
			bgSetup = new DefaultServerSetup();
			bgSetup.setValue( "db.driver", "com.mysql.jdbc.Driver" );            								
			bgSetup.setValue( "db.url", "jdbc:mysql://127.0.0.1/bgbilling?useUnicode=true&characterEncoding=Cp1251&allowUrlInLocalInfile=true&zeroDateTimeBehavior=convertToNull&dbcCompliantTruncation=false" );
			bgSetup.setValue( "db.user", "bill" );
			bgSetup.setValue( "db.pswd", "bgbilling" );
			bgSetup.initConnectionPool();
			
			EventProcessor.initProcessor( bgSetup, false );
			
			ConvertUtil.removeContracts( GROUP_ID, bgSetup);			
			
			List< Map > numbers =  loadData( args [0] );
			Map < Integer, Contract> contracts = createContacts(  numbers );
			createPoints( numbers, contracts ); 
			
			System.out.println("ok");
			
			
			
			
		} catch (Exception e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
	}
	
	
	private static class NumberChange
	{
	    public static final int REMOVED = 1;
	    public static final int DATE_CHANGED = 2;
	    public static final int ADDED = 3;
	    public static final int ANOTHER_CHANGE = 4;
	    
	    public String number;
	    public int changeType;
	    public NumberResource resource;
	    public Calendar oldDate1;
	    public Calendar oldDate2;
	    public Calendar newDate1;
	    public Calendar newDate2;
	}	

}


